import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { Text } from "./Text";
import { CloseButton } from "./IconButton";
import { useTheme } from "./Theme/Providers";
import { useFloating, useClick, useDismiss, useRole, useInteractions, FloatingPortal, FloatingFocusManager, FloatingOverlay, useId, useMergeRefs } from "@floating-ui/react";
export function useDialog({ initialOpen = false, modal = true, open: controlledOpen, onOpenChange: setControlledOpen } = {}) {
    const [uncontrolledOpen, setUncontrolledOpen] = React.useState(initialOpen);
    const [labelId, setLabelId] = React.useState();
    const [descriptionId, setDescriptionId] = React.useState();
    const open = controlledOpen ?? uncontrolledOpen;
    const setOpen = setControlledOpen ?? setUncontrolledOpen;
    const data = useFloating({
        open,
        onOpenChange: setOpen
    });
    const context = data.context;
    const click = useClick(context, {
        enabled: controlledOpen == null
    });
    //确保不会因为点击对话框区域之外立刻关闭地：//modal={state.modal}
    const dismiss = useDismiss(context, {
        outsidePressEvent: "mousedown",
        outsidePress: !modal,
    });
    // const dismiss = useDismiss(context, { outsidePressEvent: "mousedown" });
    const role = useRole(context);
    const interactions = useInteractions([click, dismiss, role]);
    return React.useMemo(() => ({
        open,
        setOpen,
        ...interactions,
        ...data,
        labelId,
        descriptionId,
        setLabelId,
        setDescriptionId,
        modal,
    }), [open, setOpen, modal, interactions, data, labelId, descriptionId]);
}
const DialogContext = React.createContext(null);
//旧版本的 useDialogState
export const useDialogContext = () => {
    const context = React.useContext(DialogContext);
    if (context == null) {
        throw new Error("Dialog components must be wrapped in <Dialog />");
    }
    return context;
};
export function Dialog({ children, ...options }) {
    const dialog = useDialog(options);
    return (_jsx(DialogContext.Provider, { value: dialog, children: children }));
}
export const DialogTrigger = ({ children, asChild = false, ref, ...props }) => {
    const context = useDialogContext();
    const childrenRef = children.props.ref; //const childrenRef = (children as any).ref;
    const refnew = useMergeRefs([context.refs.setReference, ref, childrenRef]);
    // `asChild` allows the user to pass any element as the anchor
    if (asChild && React.isValidElement(children)) {
        return React.cloneElement(children, context.getReferenceProps({
            ref: refnew,
            ...props,
            ...children.props,
            "data-state": context.open ? "open" : "closed"
        }));
    }
    return (_jsx("button", { ref: refnew, "data-state": context.open ? "open" : "closed", ...context.getReferenceProps(props), children: children }));
};
export const DialogContent = ({ layStyle, ref, ...props }) => {
    const { context: floatingContext, ...context } = useDialogContext();
    const refnew = useMergeRefs([context.refs.setFloating, ref]);
    const theme = useTheme();
    if (!floatingContext.open)
        return null;
    return (_jsx(FloatingPortal, { children: _jsx(FloatingOverlay, { className: "Dialog-overlay", lockScroll: true, style: {
                zIndex: theme.zIndices.modal,
                display: 'flex',
                overflow: 'unset',
                overflowX: 'hidden',
                ...layStyle,
            }, children: _jsx(FloatingFocusManager, { context: floatingContext, 
                // modal={floatingContext.modal}
                closeOnFocusOut: false, guards: true, returnFocus: true, children: _jsx("div", { className: "DialogContent", ref: refnew, "aria-labelledby": context.labelId, "aria-describedby": context.descriptionId, ...context.getFloatingProps(props), style: {
                        display: 'flex',
                        margin: 'auto',
                        overflowY: 'auto',
                        //这上[theme.mediaQueries.md]: {}的会报错! width放在style={{里导致无法从外面注入到这底下的css={{来覆盖！
                    }, children: _jsx("div", { className: "DialogIn", style: {
                            backgroundColor: 'white',
                            minHeight: "30vh",
                        }, css: {
                            width: "100%",
                            border: '2px solid #ddd',
                            borderRadius: theme.radii.sm,
                            boxShadow: theme.shadows.sm,
                            padding: '4px',
                            [theme.mediaQueries.md]: {
                                margin: '1rem',
                                border: '3px solid #ddd',
                                padding: '8px',
                                maxWidth: "760px",
                                boxShadow: theme.shadows.md,
                                borderRadius: theme.radii.md,
                            },
                            [theme.mediaQueries.lg]: {
                                maxWidth: "990px",
                                boxShadow: theme.shadows["lg"],
                                borderRadius: theme.radii.lg,
                            },
                            "@media print": {
                                boxShadow: 'unset',
                            },
                            ...props.style,
                        }, children: props.children }) }) }) }) }));
};
/*
这底下添加tabIndex={0}才能消除了此毛病：内容超过一个屏幕高度的话，就感觉古怪，直接跳到最后，无法正面显示头部，还要滚动。
* */
export const DialogHeading = ({ children, asChild = false, closeStyle, ref, ...props }) => {
    const { setLabelId, setOpen } = useDialogContext();
    const id = useId();
    // Only sets `aria-labelledby` on the Dialog root element
    // if this component is mounted inside it.
    React.useLayoutEffect(() => {
        setLabelId(id);
        return () => setLabelId(undefined);
    }, [id, setLabelId]);
    //支持更自由的自定义头部：
    const textComp = (asChild && React.isValidElement(children)) ?
        React.cloneElement(children, { ...children.props, })
        :
            _jsx(Text, { wrap: true, variant: "h5", children: children });
    return (_jsxs("div", { ref: ref, id: id, tabIndex: -1, autoFocus: false, style: {
            justifyContent: 'space-between',
            display: 'flex',
        }, ...props, children: [textComp, _jsx(CloseButton, { style: {
                    ...closeStyle,
                }, onClick: () => setOpen(false) })] }));
};
export const DialogDescription = ({ children, ref, ...props }) => {
    const { setDescriptionId } = useDialogContext();
    const id = useId();
    // Only sets `aria-describedby` on the Dialog root element
    // if this component is mounted inside it.
    React.useLayoutEffect(() => {
        setDescriptionId(id);
        return () => setDescriptionId(undefined);
    }, [id, setDescriptionId]);
    return (_jsx("div", { ...props, ref: ref, id: id, children: children }));
};
//原本类型:React.ButtonHTMLAttributes<HTMLButtonElement>
export const DialogClose = ({ children, ref, ...props }) => {
    const { setOpen } = useDialogContext();
    return (_jsx("button", { ...props, ref: ref, onClick: () => setOpen(false), css: {
            marginTop: '0.5rem',
            marginLeft: '1rem',
        }, children: children }));
    // return (<button type="button" {...props} ref={ref} onClick={() => setOpen(false)}/>);
};
//# sourceMappingURL=Dialog.js.map