import { jsx as _jsx } from "@emotion/react/jsx-runtime";
import * as React from "react";
// import { useTheme } from "./Theme/Providers";
// import { mergeRefs } from "./Hooks/merge-refs";
import { useFloating, autoUpdate, offset, flip, shift, useClick, useDismiss, useRole, useInteractions, FloatingPortal, FloatingFocusManager, useId, useMergeRefs } from "@floating-ui/react";
import { useTheme } from "./Theme/Providers";
export function usePopover({ initialOpen = false, placement = "bottom", modal, open: controlledOpen, onOpenChange: setControlledOpen } = {}) {
    const [uncontrolledOpen, setUncontrolledOpen] = React.useState(initialOpen);
    const [labelId, setLabelId] = React.useState();
    const [descriptionId, setDescriptionId] = React.useState();
    const open = controlledOpen ?? uncontrolledOpen;
    const setOpen = setControlledOpen ?? setUncontrolledOpen;
    const data = useFloating({
        placement,
        open,
        onOpenChange: setOpen,
        whileElementsMounted: autoUpdate,
        middleware: [
            offset(5),
            flip({
                crossAxis: placement.includes("-"),
                fallbackAxisSideDirection: "end",
                padding: 5
            }),
            shift({ padding: 5 })
        ]
    });
    const context = data.context;
    const click = useClick(context, {
        enabled: controlledOpen == null
    });
    const dismiss = useDismiss(context);
    const role = useRole(context);
    const interactions = useInteractions([click, dismiss, role]);
    return React.useMemo(() => ({
        open,
        setOpen,
        ...interactions,
        ...data,
        modal,
        labelId,
        descriptionId,
        setLabelId,
        setDescriptionId
    }), [open, setOpen, interactions, data, modal, labelId, descriptionId]);
}
const PopoverContext = React.createContext(null);
//原名是 usePopoverState
export const usePopoverContext = () => {
    const context = React.useContext(PopoverContext);
    if (context == null) {
        throw new Error("Popover components must be wrapped in <Popover />");
    }
    return context;
};
/*为适应横屏手机：限制内容高度，不应该超过4行的输入框高度。
类似Dialog 但一般页面占用尺寸较小的。默认点击外部关闭。默认跟随ref定位的。
旧的Sheet不能自适应也不配套TwoHalfFrame已经改用Popover替代;
遇到高度超出的，在html<body/>要加 overflow-y: auto; 若内容height太大自适应滚动就失去感觉了？还是BUG。
【特别注意】内容太大的，高度超出横屏手机屏幕height的情况，一定请改用Dialog; 高度超了Popover可能会出问题。
若没有加<PopoverRefer></PopoverRefer>导致缺省定位屏幕左上角,此时placement无效。高度大的不加<PopoverRefer滚动好控制。
* */
export function Popover({ children, modal = false, ...restOptions }) {
    // This can accept any props as options, e.g. `placement`,
    // or other positioning options.
    const popover = usePopover({ modal, ...restOptions });
    return (_jsx(PopoverContext.Provider, { value: popover, children: children }));
}
//保留 PopoverRefer 替代PopoverTrigger名称。 state改名context；
export const PopoverRefer = ({ children, asChild = false, ref, ...props }) => {
    const context = usePopoverContext();
    // const childrenRef = (children as any).ref;
    const refnew = useMergeRefs([context.refs.setReference, ref]); //, childrenRef
    // 使用 React.isValidElement 来检查 children 是否是一个 React 元素
    const safeChildren = React.Children.toArray(children).find(child => React.isValidElement(child));
    // 确保 safeChildren 存在并且有 props 属性
    const elementProps = (safeChildren ? safeChildren.props : {});
    //...children.props   TS2698: Spread types may only be created from object types.
    //自定义版：  `asChild` allows the user to pass any element as the anchor
    if (asChild && React.isValidElement(children)) {
        return React.cloneElement(children, context.getReferenceProps({
            ref: refnew,
            ...props,
            ...elementProps,
            "data-state": context.open ? "open" : "closed"
        }));
    }
    //正常的按钮版；
    return (_jsx("button", { ref: refnew, type: "button", "data-state": context.open ? "open" : "closed", ...context.getReferenceProps(props), children: children }));
};
export const PopoverContent = ({ style, ref, ...props }) => {
    const { context: floatingContext, ...context } = usePopoverContext();
    const refnew = useMergeRefs([context.refs.setFloating, ref]);
    const theme = useTheme();
    if (!floatingContext.open)
        return null;
    //TS2353: Object literal may only specify known properties, and '"@media print"' does not exist in type 'Properties<string | number, string & {}>'.
    return (_jsx(FloatingPortal, { children: _jsx(FloatingFocusManager, { context: floatingContext, modal: context.modal, children: _jsx("div", { className: "PopoverContent", ref: refnew, style: {
                    zIndex: theme.zIndices.popover,
                    // position: state.strategy,
                    // top: state.y ?? 0,
                    // left: state.x ?? 0,
                    backgroundColor: 'white',
                    boxShadow: theme.shadows.sm,
                    border: '1px solid #ddd',
                    padding: '4px 8px',
                    borderRadius: '4px',
                    "@media print": {
                        boxShadow: 'unset',
                    },
                    ...context.floatingStyles, ...style
                }, "aria-labelledby": context.labelId, "aria-describedby": context.descriptionId, ...context.getFloatingProps(props), children: props.children }) }) }));
};
export const PopoverHeading = ({ ref, ...props }) => {
    const { setLabelId } = usePopoverContext();
    const id = useId();
    // Only sets `aria-labelledby` on the Popover root element
    // if this component is mounted inside it.
    React.useLayoutEffect(() => {
        setLabelId(id);
        return () => setLabelId(undefined);
    }, [id, setLabelId]);
    return (_jsx("h4", { ...props, ref: ref, id: id, children: props.children }));
};
export const PopoverDescription = ({ ref, ...props }) => {
    const { setDescriptionId } = usePopoverContext();
    const id = useId();
    // Only sets `aria-describedby` on the Popover root element
    // if this component is mounted inside it.
    React.useLayoutEffect(() => {
        setDescriptionId(id);
        return () => setDescriptionId(undefined);
    }, [id, setDescriptionId]);
    return (_jsx("div", { className: "PopoverDescription", ...props, ref: ref, id: id, style: {
            display: "flex",
            flexWrap: 'wrap',
        } }));
    // return <p {...props} ref={ref} id={id} />;
};
//原本类型React.ButtonHTMLAttributes<HTMLButtonElement>
export const PopoverClose = ({ ref, ...props }) => {
    const { setOpen } = usePopoverContext();
    return (_jsx("button", { type: "button", style: {
            display: "block",
            margin: 'auto',
        }, ref: ref, ...props, onClick: (event) => {
            props.onClick?.(event);
            setOpen(false);
        } }));
};
//# sourceMappingURL=Popover.js.map